package fr.velossity.sample.adapter.device.impl;

import java.util.Dictionary;
import java.util.Hashtable;

import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.osgi.framework.ServiceRegistration;
import org.osgi.util.tracker.ServiceTracker;

import fr.velossity.sample.adaptee.AdapteeDevice;
import fr.velossity.sample.device.Device;

/**
 * This factory is responsible to create dynamically a Device adapter when a low
 * device is appearing and to delete it when it is disappearing. It is using a simple
 * tracker for discovering low devices.
 * 
 * @author C. Saint-Marcel
 * 
 */
public class BundleActivatorImpl implements BundleActivator {
	
	/**
	 * A reference to the low device tracker.
	 */
	private AdapteeTracker adapteeTracker;
	
	/**
	 * The tracker extension. 
	 */
	private class AdapteeTracker extends ServiceTracker {
	    public AdapteeTracker(BundleContext ctx) {
	        super(ctx, AdapteeDevice.class.getName(), null);
	    }

	    /**
	     * @see ServiceTracker#addingService(ServiceReference)
	     */
		public Object addingService(ServiceReference ref) {
	        AdapteeDevice adaptee = (AdapteeDevice) context.getService(ref);
	        Device adapter = new DeviceAdapterImpl(adaptee);
	        
	        // REGISTERS with mandatory properties 
	        Dictionary properties = new Hashtable();
			properties.put(Device.DEVICE_PROPERTY_IDENTIFIER, adapter.getIdentifier());
			properties.put(Device.DEVICE_PROPERTY_TYPE, adapter.getType());
			return context.registerService(Device.class.getName(), adapter, properties);
	    }
	    
		/**
		 * @see ServiceTracker#removedService(ServiceReference, Object)
		 */
	    public void removedService(ServiceReference ref, Object svc) {
	        ServiceRegistration reg = (ServiceRegistration) svc;
	        reg.unregister();
	        context.ungetService(ref);
	    }
	}

	/**
	 * @see BundleActivator#start(BundleContext)
	 */
	public void start(BundleContext context) throws Exception {
		adapteeTracker = new AdapteeTracker(context);
		adapteeTracker.open();
		
	}

	/**
	 * @see BundleActivator#stop(BundleContext)
	 */
	public void stop(BundleContext context) throws Exception {
		adapteeTracker.close();
	}
}
